/*
Units V1.1 - A general units converter
Copyright (C) 1998 Franois Pessaux (francois.pessaux@inria.fr)


This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include <Common.h>
#include <System/SysAll.h>
#define _DONT_USE_FP_TRAPS_ 1
#include <System/NewFloatMgr.h>
#include <UI/UIAll.h>
#include "gccfloats.h"
#include "protos.h"
#include "messages.h"

#include "pilrc.h"
#include "units.h"
#include "bool.h"


extern DmOpenRef unit_db ;
extern Word CurrentCategory ;
extern char result_buffer[] ;

extern FormPtr MainForm ;
extern ListPtr TopList, BottomList, TopPrefList, BottomPrefList ;
extern FieldPtr TopField, BottomField ;
extern Boolean pretty_floats ;


#define maxShift 5


/* Try to pretty print floats without scientific notation if possible */
void PrettyOutput (char *initial_buffer)
{
  if (pretty_floats) {
    UInt len ;
    short exp ;
    short shift ;
    char *tmp ;
    unsigned char i ;

    /* Result is 0 so do nothing */
    if (initial_buffer[0] == '\0') return ;

    len = StrLen (initial_buffer) ;
    exp = initial_buffer[len-1] - '0' ;
    exp += (initial_buffer[len-2] - '0') * 10 ;
    if (initial_buffer[len-3] == '-') exp = -exp ;

    if (exp >= 0) {
      /* Positive (or null) exponent. We must shift dot to right */
      shift = min (exp, maxShift) ;
      /* Look for the dot */
      tmp = StrChr (initial_buffer, '.') ;

      /* Shift characters */
      for (i = 0; i < shift; i++) tmp[i] = tmp[i+1] ;

      /* Put the dot */
      tmp[i] = '.' ;

      /* Look for 'e' */
      tmp = StrChr (initial_buffer, 'e') ;

      /* Catenate exponent if needed */
      exp -= shift ;
      if (exp > 0) StrPrintF (tmp, "/%d", exp) ;
      else *tmp = '\0' ;
    }
    else {
      /* Negative exponent */
      /* Do nothing for the moment */
      tmp = StrChr (initial_buffer, 'e') ;
      *tmp = '\0' ;
      len = StrLen (initial_buffer) ;
      i = len - 1 ;
      while (initial_buffer[i] == '0') i-- ;
      shift = len - i - 1 ;
      /* # time we have to shift */
      shift = min (shift, -exp) ;
      if (shift <= 0) {
	/* Restore string and change 'e' to fit font 6 */
	*tmp = '/' ;
	return ;
      }

      /* Shift characters */
      for (i = len-1-shift; initial_buffer[i] != '.' ; i--)
	initial_buffer[i+shift] = initial_buffer[i] ;

      /* Fetch the old first significant digit */
      initial_buffer[i+shift] = initial_buffer[i-1] ;
      /* We must pad from i+shift until dot with '0' */
      for (i = i+shift-1; initial_buffer[i] != '.'; i--)
	initial_buffer[i] = '0' ;
      /* And then first significant digit is alway '0' */
      /* At his stage, i always point to the '.' char */
      initial_buffer[i-1] = '0' ;

      /* Catenate exponent if needed */
      exp += shift ;
      if (exp < 0) StrPrintF (tmp, "/%d", exp) ;
    }
  }
  else {
    /* No pretty printing. Use scientific notation */
    /* We just have to change 'e' by 47 to  get a 'e' in font 6 */
    CharPtr found ;

    found = StrChr (initial_buffer, 'e') ;
    if (found) *found = 47 ;
  }
}


/* Calcule x */
double power (double x, short y)
{
  double result = 1.0 ;

  if (y > 0) {
    for (;y > 0; y--) result = GccSysFlpMul (result, x) ;
  }
  else {
    if (y < 0) {
      for (;y < 0; y++) result = GccSysFlpDiv (result, x) ;
    }
  }
return (result) ;
}




void Convert (void)
{
  CharPtr SrcUnitName, DestUnitName, SrcPrefixName, DestPrefixName, tmp_str ;
  double Src_A, Src_B, Src_M, Dest_A, Dest_B, Dest_M, result ;
  unsigned short Src_Prefixes, Dest_Prefixes ;
  Word Src_category, Dest_category ;

  /* Il faut aller recuperer l'unite de depart */
  SrcUnitName = LstGetSelectionText (TopList, LstGetSelection (TopList)) ;
  /* Il faut aller recuperer l'unite d'arrivee */
  DestUnitName = LstGetSelectionText(BottomList,LstGetSelection (BottomList)) ;
  /* On recupere les prefixes */
  SrcPrefixName = LstGetSelectionText (TopPrefList,
				       LstGetSelection (TopPrefList)) ;
  DestPrefixName = LstGetSelectionText (BottomPrefList,
					LstGetSelection (BottomPrefList)) ;

  if ((!SrcUnitName) || (!DestUnitName)) {
    /* L'utilisateur n'a pas specifie d'unites pour la conversion */
    FrmCustomAlert (standardErrorAlert, MSG_MISSUNIT, NULL, NULL) ;
    return ;
  }

  /* Maintenant, on a les abbrevs des unites de src et dest */
  /* Il faut aller rechercher les facteurs */
  Src_category = CurrentCategory ; /* Pour dire de chercher dans la */
  Dest_category = CurrentCategory ;  /* categorie courante */
  FindFactorsByAbbrevName (SrcUnitName, &Src_A, &Src_B, &Src_M,
			   &Src_Prefixes, &Src_category) ;
  FindFactorsByAbbrevName (DestUnitName, &Dest_A, &Dest_B, &Dest_M,
			   &Dest_Prefixes, &Dest_category) ;
  if (Src_category != Dest_category) {
    /* Erreur de "type" de conversion */
    FrmCustomAlert (standardErrorAlert,
		    MSG_UNCONSISTENT_CONVERSION, NULL, NULL) ;
    return ;
  }
  /* Conversion src -> reference */
  tmp_str = FldGetTextPtr (TopField) ;
  if ((!tmp_str) || (!tmp_str[0])) {
    /* L'utilisateur a oublie de donner des informations */
    FrmCustomAlert (standardErrorAlert,
		    MSG_MISSING, NULL, NULL) ;
    return ;
  }

  result = GccSysFlpAToF (tmp_str) ;
  /* result = (result * Src_A * Src_M^Prefix) + Src_B ; */
  result = GccSysFlpMul (result, Src_A) ;
  result = GccSysFlpMul (result,
			 power (Src_M, PrefixCoefFromName (SrcPrefixName))) ;
  result = GccSysFlpAdd (result, Src_B) ;

  /* result = (result - Dest_B) / Dest_A / Dest_M^Prefix ; */
  result = GccSysFlpSub (result, Dest_B) ;
  result = GccSysFlpDiv (result, Dest_A) ;
  result = GccSysFlpDiv (result,
  			 power (Dest_M, PrefixCoefFromName (DestPrefixName))) ;
  /* Il faut mettre a jour le field resultat */
  GccSysFlpFToA (result, result_buffer) ;
  PrettyOutput (result_buffer) ;

  FldSetTextPtr (BottomField, result_buffer) ;
  FldRecalculateField (BottomField, true) ;

  FrmDrawForm (MainForm) ;
}

