/*
Units V1.1 - A general units converter
Copyright (C) 1998 Franois Pessaux (francois.pessaux@inria.fr)


This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include <Common.h>
#include <System/SysAll.h>
#define _DONT_USE_FP_TRAPS_ 1
#include <System/NewFloatMgr.h>
#include <UI/UIAll.h>
#include "gccfloats.h"
#include "protos.h"


#include "pilrc.h"
#include "units.h"
#include "bool.h"


extern DmOpenRef unit_db ;


/* Recherche le record de nom 'name'. Retrourne son index par effet */
/* de bord. Retourne TRUE si on l'a trouve, sinon FALSE. C'est seulement */
/* dans le premier cas que 'found_index' est significatif */
/* Met a jour 'found_index' seulement s'il n'est pas nul */
Boolean FindUnitRecordIndexByName (CharPtr name, UInt *found_index,
				   Word InCategory)
{
  UInt index ;
  UInt i ;
  VoidHand handle ;
  VoidPtr record ;
  UInt units_number ;

  units_number = DmNumRecordsInCategory (unit_db, InCategory) ;
  for(i=0, index=0; i<units_number; i++, index++) {
    handle = DmQueryNextInCategory (unit_db, &index, InCategory) ;
    record = MemHandleLock (handle) ;
    if (StrCompare (name, &((CharPtr)record)[nameDataOffset]) == 0) {
      if (found_index) *found_index = index ;
      MemHandleUnlock (handle) ;
      DmReleaseRecord (unit_db, index, FALSE) ;
      return (TRUE) ;
    }
    MemHandleUnlock (handle) ;
    DmReleaseRecord (unit_db, index, FALSE) ;
  }

  /* Si on arrive ici c'est qu'on n'a rien trouve */
  return (FALSE) ;
}


/* Retourne TRUE si on a trouve les infos cherchee, FALSE sinon. */
/* Rend son resultat par effet de bord dans les 3 pointeurs */
/* passes en argument. La veleur passee dans categ sert a dire */
/* dans quelle categorie rechercher. Si c'est dans "all", et qu'on */
/* a trouve, alors categ sera mis a jour avec la categorie reelle */
/* d'appartenance du record */
/* Met a jour factor_a et factor_b s'ils ne sont pas a NULL */
Boolean FindFactorsByAbbrevName (CharPtr name, double *factor_a,
				 double *factor_b, double *factor_m,
				 unsigned short *prefixes, Word *categ)
{
  UInt index ;
  VoidHand handle ;
  VoidPtr record ;
  UInt record_attr ;

  if (FindUnitRecordIndexByName (name, &index, *categ)) {
    handle = DmQueryRecord (unit_db, index) ;
    record = MemHandleLock (handle) ;
    if (factor_a) *factor_a = ((struct UnitRecordHeader*)record)->afactor ;
    if (factor_b) *factor_b = ((struct UnitRecordHeader*)record)->bfactor ;
    if (factor_m) *factor_m = ((struct UnitRecordHeader*)record)->mfactor ;
    if (prefixes) *prefixes = ((struct UnitRecordHeader*)record)->prefixes ;
    /* On recupere l'info header du record pour trouver */
    /* a quelle categorie il appartient.                */
    DmRecordInfo (unit_db, index, &record_attr, NULL, NULL) ;
    *categ = record_attr & dmRecAttrCategoryMask ;
    MemHandleUnlock (handle) ;
    DmReleaseRecord (unit_db, index, FALSE) ;
    return (TRUE) ;
    }

    /* Si on arrive ici c'est qu'on n'a rien trouve */
    return (FALSE) ;
}

