/*
Units V1.1 - A general units converter
Copyright (C) 1998 Franois Pessaux (francois.pessaux@inria.fr)


This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include <Common.h>
#include <System/SysAll.h>
#include <System/NewFloatMgr.h>
#include <UI/UIAll.h>
#include "protos.h"


#include "units.h"
#include "bool.h"


extern DmOpenRef unit_db ;
extern AppInfoType appinfo ;
extern Word CurrentCategory ;
extern Word last_saved_src, last_saved_dest ;
extern Word last_saved_pref_src, last_saved_pref_dest;
extern Boolean pretty_floats ;


/* Necessaire au cas ou on rentre une base de donnee non triee et */
/* pour inserer les records tries quand on cree ou modifie une unite */
Int DbSort (void *rec1, void *rec2, Int other,
	    SortRecordInfoPtr si1, SortRecordInfoPtr si2,
	    VoidHand appinfoh)
{
return (
	StrCompare (&((unsigned char*)rec1)[nameDataOffset],
		    &((unsigned char*)rec2)[nameDataOffset])) ;
}




/* Return TRUE if successful */
Boolean InitApplication (void)
{
  LocalID db_id ;        /* Local Id de la database */
  UInt db_attrs ;
  LocalID appInfoID ;

  /* On recherche le local ID de la database */
  db_id = DmFindDatabase (0, DB_NAME) ;
  if (!db_id) {
    /* La database n'existe pas, on la cree */
    DmCreateDatabase (0, DB_NAME, CAT_APP_ID, CAT_DB_TYPE, FALSE) ;
    db_id = DmFindDatabase (0, DB_NAME) ;
  }

  /* On ouvre la database */
  unit_db = DmOpenDatabase (0, DmFindDatabase (0, DB_NAME), dmModeReadWrite) ;
  if (!unit_db) return (FALSE) ;

  /* On va verifier et creer si necessaire le header de la database */
  /* au passage, on recupere les attributs de la database */
  DmDatabaseInfo (0, db_id, NULL, &db_attrs, NULL, NULL, NULL, 
		  NULL, NULL, &appInfoID, NULL, NULL, NULL) ;
  if (appInfoID == NULL) {
    VoidHand h ;
    AppInfoType *appInfoP ;

    /* Le header AppInfo n'existe pas, il faut donc l'ecrire */
    /* On s'alloue une zone de memoire de la taille de ce header. On */
    /* l'alloue directement dans le meme heap que la database, comme */
    /* ca on pourra directement l'accrocher a la database.           */
    h = DmNewHandle (unit_db, sizeof (AppInfoType)) ;
    appInfoID = MemHandleToLocalID (h) ;
    /* On accroche la zone de mem qui va contenir ce header a la database */
    DmSetDatabaseInfo (0, db_id, NULL, NULL, NULL, NULL, NULL, NULL,
		       NULL, &appInfoID, NULL, NULL, NULL) ;
    /* On locke cette zone de memoire */
    appInfoP = MemLocalIDToLockedPtr (appInfoID, 0) ;
    /* On ecrit notre header dans la memoire. Par effet de bord, ca sera */
    /* fixe dans la database. */
    DmWrite(appInfoP, 0, &appinfo, sizeof (AppInfoType)) ;
    /* De-verrouillage de la memoire */
    MemPtrUnlock(appInfoP) ;
  }

  /* On marque la base de donnee comme etant modifiee */
  /* ainsi, la HotSync backupera a coup sur la database */
  db_attrs = db_attrs | dmHdrAttrBackup ;
  DmSetDatabaseInfo (0, db_id, NULL, &db_attrs, NULL, NULL, NULL,
		     NULL, NULL, NULL, NULL, NULL, NULL) ;

  /* Necessaire au cas ou on rentre une base de donnee non triee */
  /* DmQuickSort (unit_db, DbSort, 0) ; */

  /* On charge les preferences */
  { struct UnitPreferences prefs ;
    char tmp_category_name[dmCategoryLength] ;
    UInt record_index ;
  if (PrefGetAppPreferencesV10 (CAT_APP_ID, 0, &prefs,
				sizeof (struct UnitPreferences))) {
      pretty_floats =  prefs.pretty_floats ;
      CategoryGetName (unit_db, prefs.current_category, tmp_category_name) ;
      /* On ne se repositionne sur la derniere categorie que si elle */
      /* existe toujours. Toua au moins, que si il en existe une de  */
      /* meme index. */
      if (tmp_category_name[0] != '\0') {
	CurrentCategory = prefs.current_category ;

	/* Fetch uniques ID of saved active unit only if significative */
	if (prefs.ids_are_valid) {
	  /* Recherche le numero du record src dans la categorie */
	  if (!DmFindRecordByID (unit_db, prefs.src_unique_id, &record_index))
	    last_saved_src = DmPositionInCategory
                              (unit_db, record_index, CurrentCategory) ;

	  /* Recherche le numero du record dest dans la categorie */
	  if (!DmFindRecordByID (unit_db, prefs.dest_unique_id, &record_index))
	    last_saved_dest = DmPositionInCategory
	                       (unit_db, record_index, CurrentCategory) ;
	  /* Get prefixes used when we switched application */
	  last_saved_pref_src = prefs.src_prefix ;
	  last_saved_pref_dest = prefs.dest_prefix ;
	}
      }
    }
  }

  return (TRUE) ;
}
