/*
Units V1.1 - A general units converter
Copyright (C) 1998 Franois Pessaux (francois.pessaux@inria.fr)


This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include <Common.h>
#include <System/SysAll.h>
#define _DONT_USE_FP_TRAPS_ 1
#include <System/NewFloatMgr.h>
#include <UI/UIAll.h>
#include "gccfloats.h"
#include "protos.h"
#include "messages.h"

#include "pilrc.h"
#include "units.h"
#include "bool.h"


extern DmOpenRef unit_db ;
extern Word CurrentCategory ;
extern char CurrentCategoryName[dmCategoryLength] ;
extern Word UnitsNbEntries ;
extern char **UnitsLstItems ;
extern unsigned char selection_continuation ;


/* Mis a jour juste avant d'appeler la creation/modification d'unite */
/* par la form de selection d'unite */
extern UInt record_to_modify_index ;



Boolean NewUnitEventHandler (EventPtr event)
{
  FormPtr form ;
  double floatvalA, floatvalB, floatvalM ;
  VoidHand handle ;
  VoidPtr record ;
  UInt index ;
  FieldPtr afactor_field, bfactor_field, mfactor_field, name_field ;
  CharPtr afact_text, bfact_text, mfact_text, name_text ;
  unsigned char name_len ;
  UInt record_attr ;
  ControlPtr trigger ;
  unsigned short Prefixes ;
  char i ;
  ControlPtr chkbox ;

  switch (event->eType) 
    {
    case frmOpenEvent:
      {
      Handle afactor_handle, bfactor_handle, mfactor_handle, name_handle ;

      form = FrmGetActiveForm () ;

      name_handle = (Handle) MemHandleNew (25) ;
      afactor_handle = (Handle) MemHandleNew (17) ;
      bfactor_handle = (Handle) MemHandleNew (17) ;
      mfactor_handle = (Handle) MemHandleNew (17) ;

      /* Maintenant, si on est en mode de modification d'unite, il faut */
      /* initialiser l'interface avec les donnees */
      if (selection_continuation == CONTINUATION_MODIFY) {
	VoidPtr mem_zone ;

	/* Les infos se trouvent dans le record 'record_to_modify_index' */
	handle = DmGetRecord (unit_db, record_to_modify_index) ;
	record = MemHandleLock (handle) ;

	/* La categorie courante devient celle de l'unite modifiee */
	DmRecordInfo (unit_db, record_to_modify_index,
		      &record_attr, NULL, NULL) ;
	CurrentCategory = record_attr & dmRecAttrCategoryMask ;

	/* Recopie du nom */
	mem_zone = MemHandleLock ((VoidHand) name_handle) ;
	StrCopy ((unsigned char*)mem_zone,
		 &((unsigned char*)record)[nameDataOffset]) ;
	MemHandleUnlock ((VoidHand) name_handle) ;

	/* Recopie du facteur a */
	mem_zone = MemHandleLock ((VoidHand) afactor_handle) ;
	GccSysFlpFToA (((struct UnitRecordHeader*)record)->afactor ,mem_zone) ;
	MemHandleUnlock ((VoidHand) afactor_handle) ;

	/* Recopie du facteur b */
	mem_zone = MemHandleLock ((VoidHand) bfactor_handle) ;
	GccSysFlpFToA (((struct UnitRecordHeader*)record)->bfactor ,mem_zone) ;
	MemHandleUnlock ((VoidHand) bfactor_handle) ;

	/* Recopie du facteur m */
	mem_zone = MemHandleLock ((VoidHand) mfactor_handle) ;
	GccSysFlpFToA (((struct UnitRecordHeader*)record)->mfactor ,mem_zone) ;
	MemHandleUnlock ((VoidHand) mfactor_handle) ;

	/* Recopie de l'etat des checkboxes */
	Prefixes = ((struct UnitRecordHeader*)record)->prefixes ;
	for (i=0;i<16;i++) {
	  /* ASSUME que le checkbox du plus petit prefixe est celui */
	  /* qui a le plus petit ID et que les ID de prefixes sont */
	  /* ranges en ordre croissant consecutifs */
	  chkbox = FrmGetObjectPtr (form,
				    FrmGetObjectIndex (form, i+newUnitaChk)) ;
	  CtlSetValue (chkbox, (Prefixes >> i) & 0x1) ;
	}

	/* Liberation du record de la base de donnee */
	MemHandleUnlock (handle) ;
	DmReleaseRecord (unit_db, record_to_modify_index, FALSE) ;
      }
      else {
	/* On clos les chaines des fields pour eviter */
	/* d'avoir des caracteres bizares */
	record = MemHandleLock ((VoidHand) name_handle) ;
	((unsigned char*)record)[0] = '\0' ;
	MemHandleUnlock ((VoidHand) name_handle) ;
	record = MemHandleLock ((VoidHand) afactor_handle) ;
	((unsigned char*)record)[0] = '\0' ;
	MemHandleUnlock ((VoidHand) afactor_handle) ;
	record = MemHandleLock ((VoidHand) bfactor_handle) ;
	/* Init par default pour le champ 'b' */
	StrCopy ((unsigned char*)record, "0") ;
	MemHandleUnlock ((VoidHand) bfactor_handle) ;
	record = MemHandleLock ((VoidHand) mfactor_handle) ;
	/* Init par default pour le champ 'm' */
	StrCopy ((unsigned char*)record, "10") ;
	MemHandleUnlock ((VoidHand) mfactor_handle) ;
      }

      /* On doit "initialiser" le popup trigger avec la categorie courante */
      CategoryGetName (unit_db, CurrentCategory, CurrentCategoryName) ;
      trigger = FrmGetObjectPtr (form,
				 FrmGetObjectIndex (form,
						    unitCategoryTrigger)) ;
      CategorySetTriggerLabel (trigger, CurrentCategoryName) ;

      name_field = (FieldPtr) FrmGetObjectPtr (form, FrmGetObjectIndex
				               (form, newUnitNameField)) ;
      afactor_field = (FieldPtr) FrmGetObjectPtr (form, FrmGetObjectIndex
				               (form, newUnitFactorAField)) ;
      bfactor_field = (FieldPtr) FrmGetObjectPtr (form, FrmGetObjectIndex
				               (form, newUnitFactorBField)) ;
      mfactor_field = (FieldPtr) FrmGetObjectPtr (form, FrmGetObjectIndex
				               (form, newUnitFactorMField)) ;

      /* Lier les handles aux fields */
      FldSetTextHandle (name_field, name_handle) ;
      FldSetTextHandle (afactor_field, afactor_handle) ;
      FldSetTextHandle (bfactor_field, bfactor_handle) ;
      FldSetTextHandle (mfactor_field, mfactor_handle) ;
      FrmDrawForm (form) ;

      /* On met le focus directement sur le field pour saisir le nom */
      /* de l'unite */
      FrmSetFocus (form, FrmGetObjectIndex (form, newUnitNameField)) ;
      return (TRUE) ;
      }
      break ;

    case ctlSelectEvent:
      switch (event->data.ctlEnter.controlID)
	{
	case okButton:
	  /* Il faut verifier que les coefficients a et b sont bien des */
	  /* flottants et que a est different de 0 */
	  form = FrmGetActiveForm () ;

	  name_field = (FieldPtr) FrmGetObjectPtr (form, FrmGetObjectIndex
				               (form, newUnitNameField)) ;
	  afactor_field = (FieldPtr) FrmGetObjectPtr (form, FrmGetObjectIndex
				               (form, newUnitFactorAField)) ;
	  bfactor_field = (FieldPtr) FrmGetObjectPtr (form, FrmGetObjectIndex
				               (form, newUnitFactorBField)) ;
	  mfactor_field = (FieldPtr) FrmGetObjectPtr (form, FrmGetObjectIndex
				               (form, newUnitFactorMField)) ;

	  name_text = FldGetTextPtr (name_field) ;
	  afact_text = FldGetTextPtr (afactor_field) ;
	  bfact_text = FldGetTextPtr (bfactor_field) ;
	  mfact_text = FldGetTextPtr (mfactor_field) ;

	  if ((!name_text) || (!afact_text) || (!bfact_text) ||
	      (!mfact_text) || (!name_text[0]) || (!afact_text[0]) ||
	      (!bfact_text[0]) || (!mfact_text[0])) {
	    /* L'utilisateur a oublie de donner des informations */
	    /* On ne fait rien sinon afficher un message d'erreur */
	    FrmCustomAlert (standardErrorAlert, MSG_MISSING, NULL, NULL) ;
	    return (TRUE) ;
	  }

	  /* Verification de si l'unite n'existe pas deja dans */
	  /* le cas ou on est en ajout de nouvelle unite */
	  if (selection_continuation == CONTINUATION_NONE) {
	    if (FindUnitRecordIndexByName(name_text, NULL, CurrentCategory)){
	      /* L'unite existait deja */
	      FrmCustomAlert (standardErrorAlert, MSG_EXIST, NULL, NULL) ;
	      /* On ne fait rien */
	      return (TRUE) ;
	    }
	  }
	  else {
	    /* Dans le cas ou on est en modification, il faut verifier */
	    /* que le nom n'est pas le meme que celui d'une autre unite */
	    /* que celle qu'on modifiait */
	    handle = DmGetRecord (unit_db, record_to_modify_index) ;
	    if (!handle) {
	      /* Can't get record */
	      FrmCustomAlert (standardErrorAlert,
			      MSG_CANT_GET_RECORD, NULL, NULL) ;
	      return (TRUE) ;
	    }
	    record = MemHandleLock (handle) ;
	    if (!record) {
	      /* Can't lock record to get memory access */
	      FrmCustomAlert (standardErrorAlert,
			      MSG_CANT_GET_RECORD, NULL, NULL) ;
	      return (TRUE) ;
	    }
	    if (StrCompare (name_text,
			    &((unsigned char*)record)[nameDataOffset]) != 0) {
	      if (FindUnitRecordIndexByName(name_text, NULL, CurrentCategory)){
		FrmCustomAlert (standardErrorAlert, MSG_EXIST, NULL, NULL) ;
		MemHandleUnlock (handle) ;
		DmReleaseRecord (unit_db, record_to_modify_index, FALSE) ;
		/* On ne fait rien */
		return (TRUE) ;
	      }
	    }
	    MemHandleUnlock (handle) ;
	    DmReleaseRecord (unit_db, record_to_modify_index, FALSE) ;
	  }

	  /* Facteurs a et b */
	  floatvalA = GccSysFlpAToF (afact_text) ;
	  /* Verifie l'inversion au passage */
	  chkbox = FrmGetObjectPtr (form,
				    FrmGetObjectIndex (form, inverseAChk)) ;
	  if (CtlGetValue (chkbox))
	    floatvalA = GccSysFlpDiv (1.0, floatvalA) ;

	  floatvalB = GccSysFlpAToF (bfact_text) ;
	  floatvalM = GccSysFlpAToF (mfact_text) ;

	  /* if (floatvalA == 0.0) { */
	  if (GccSysFlpEq (floatvalA, 0.0)) {
	    /* On risque la division par 0.0 ! */
	    FrmCustomAlert (standardErrorAlert, MSG_NULLA, NULL, NULL) ;
	    return (TRUE) ;
	  }

	  /* if (floatvalA == 0.0) { */
	  if (GccSysFlpEq (floatvalM, 0.0)) {
	    /* On risque la division par 0.0 ! */
	    FrmCustomAlert (standardErrorAlert, MSG_NULLM, NULL, NULL) ;
	    return (TRUE) ;
	  }

	  /* Calcul des prefixes */
	  Prefixes = 0 ;
	  for (i=0;i<16;i++) {
	    /* ASSUME que le checkbox du plus petit prefixe est celui */
	    /* qui a le plus petit ID et que les ID de prefixes sont */
	    /* ranges en ordre croissant consecutifs */
	    chkbox = FrmGetObjectPtr (form,
                             FrmGetObjectIndex (form, i+newUnitaChk)) ;
	    Prefixes |= ((unsigned short) CtlGetValue (chkbox)) << i ;
	  }

	  /* Calcul de la taille du futur record */
	  name_len = (unsigned char) StrLen (name_text) + 1 ;
	  {
	    ULong recordsize ;

	    recordsize = (ULong) (sizeof (struct UnitRecordHeader)+name_len) ;
	    /* Maintenant, on connait la taille du futur record */
	    /* On l'alloue directement dans la databse ou on va le chercher */
	    /* si on etait en modification */
	    if (selection_continuation == CONTINUATION_NONE) {
	      index = 0 ;
	      handle = DmNewRecord (unit_db, &index, recordsize) ;
	    }
	    else {
	      index = record_to_modify_index ;
	      /* Il faut retailler le record comme il faut */
	      handle = DmResizeRecord (unit_db, index, recordsize) ;
	    }
	  }
	  record = MemHandleLock (handle) ;

	  /* Maintenant, on ecrit les facteurs */
	  DmWrite (record, factAOffset, &floatvalA, sizeof (double)) ;
	  DmWrite (record, factBOffset, &floatvalB, sizeof (double)) ;
	  DmWrite (record, factMOffset, &floatvalM, sizeof (double)) ;
	  DmWrite (record, prefixesOffset, &Prefixes,
		   sizeof (unsigned short)) ;

	  /* Longueur du nom complet */
	  DmWrite (record, nameSizeOffset, &name_len, sizeof(unsigned char)) ;

	  /* Nom complet lui-meme */
	  DmWrite (record, nameDataOffset, name_text, name_len) ;

	  /* Mettre le record dans la bonne categorie */
	  DmRecordInfo (unit_db, index, &record_attr, NULL, NULL) ;
	  record_attr &= ~dmRecAttrCategoryMask ;

	  /* Si on est dans "All categories", ranger le record */
	  /* dans la categorie "Unfiled". */
	  if (CurrentCategory == dmAllCategories)
	    record_attr |= dmUnfiledCategory ;
	  else
	    record_attr |= CurrentCategory ;
	  DmSetRecordInfo (unit_db, index, &record_attr, NULL) ;

	  { UInt new_index ;
	    /* On cherche ou ranger le record pour gqrder le tri */
	    new_index = DmFindSortPosition (unit_db, record, NULL, DbSort, 0) ;
	    /* On le range */
	    DmMoveRecord (unit_db, index, new_index) ;
	  }

	  /* On libere la record */
	  MemHandleUnlock (handle) ;

	  /* Le record a ete modifie (ou cree), il est donc dirty */
	  DmReleaseRecord (unit_db, index, TRUE) ;
	  /* FALL THROUGH */

	case cancelButton:
	  FrmReturnToForm (0) ;
	  /* Ne pas desallouer les handles des fields; le systeme*/
	  /* le fait pour nous. */
	  /* Commit des eventuels changements dans les categories */
	  ReleaseUnitList (UnitsLstItems) ;
	  UnitsLstItems = BuildUnitLists (CurrentCategory, &UnitsNbEntries) ;
	  GlobalUpdateMainFormUI () ;
	  return (TRUE) ;
	  break ;

	case unitCategoryTrigger:
	  CategorySelectV10 (unit_db, FrmGetActiveForm (), unitCategoryTrigger,
			     unitCategoryList, FALSE, &CurrentCategory,
			     CurrentCategoryName) ;
     	  return (TRUE) ;
	  break ;
	}

    case menuEvent:
      switch (event->data.menu.itemID)
	{
	case menuitemCut2:
	case menuitemPaste2:
	case menuitemCopy2:
	  { Word tmp_index ;
	    FieldPtr tmp_fld ;

	    form = FrmGetActiveForm () ;
	    tmp_index = FrmGetFocus (form) ;
	    if (tmp_index != -1) {
	      /* On croise les doigts pour qu'il n'y ait que les */
	      /* fields qui puissent avoir le focus !!! */
	      tmp_fld = FrmGetObjectPtr (form, tmp_index) ;
	      switch (event->data.menu.itemID) {
	      case menuitemCut2: FldCut (tmp_fld) ; break ;
	      case menuitemPaste2: FldPaste (tmp_fld) ; break ;
	      case menuitemCopy2: FldCopy (tmp_fld) ; break ;
	      }
	    }
	  }
	  return (TRUE) ;
	  break ;
	}
    }
  return (FALSE) ;
}
