/*
Units V1.1 - A general units converter
Copyright (C) 1998 Franois Pessaux (francois.pessaux@inria.fr)


This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>
#include "protos.h"
#include "pilrc.h"
#include "units.h"


extern char prefixes_strings[16][3] ;
extern Word CurrentCategory ;
extern ControlPtr TopPrefTrigger, BottomPrefTrigger ;
extern ListPtr TopList, BottomList, TopPrefList, BottomPrefList ;
char **FromPrefixesList ;
Word FromPrefixesNumber ;
char **ToPrefixesList ;
Word ToPrefixesNumber ;
extern Word last_saved_pref_src, last_saved_pref_dest ;




/* A partir d'un nom de categorie, cree le tableau de chaines */
/* constituant la liste de ses prefixes autorises */
/* Met a jour nb_entries avec le nobre de prefixes trouves */
char** PrefixListFromUnitName (char *name, Word category, Word *nb_entries)
{
  char **string_lst ;
  unsigned short prefixes ;
  char i ;
  Word prefixes_found ;

  /* On recherche le mot contenant les prefixes */
  FindFactorsByAbbrevName (name, NULL, NULL, NULL, &prefixes, &category) ;

  /* On alloue la taille max, on re-taillera apres */
  string_lst = (char**) MemPtrNew (17*sizeof(char*)) ;

 /* Par default, on a toujours l'unite avec le prefixe vide */
  string_lst[0] = prefixes_strings[17] ;
  prefixes_found = 1 ;
  for (i=0; i<16; i++) {
    if (prefixes & (1 << i)) {
      /* Le prefixe est autorise */
      string_lst[prefixes_found] = prefixes_strings[i] ;
      prefixes_found++ ;
    }
  }

  /* Maintenant, on re-taille la liste a la bonne taille */
  MemPtrResize (string_lst, prefixes_found*sizeof(char*)) ;
  if (nb_entries) *nb_entries = prefixes_found ;

  /* Ne pas delocker le pointeur car il va resservir ! */
  return (string_lst) ;
}




/* Determine quelle unite (source ou destination) doit voir ses */
/* prefixes rafraichis en fonction de l'ID du trigger associe */
void RefreshPrefixList (Word trigger_id)
{
  char **string_lst ;
  Word *nb ;
  ListPtr list ;
  ControlPtr trigger ;
  char *unit_name ;
  Word last_used_prefix ;

  if (trigger_id == unitFromTrigger) {
    string_lst = FromPrefixesList ;
    nb = &FromPrefixesNumber ;
    list = TopPrefList ;
    trigger = TopPrefTrigger ;
    unit_name = LstGetSelectionText (TopList, LstGetSelection (TopList)) ;
    last_used_prefix = last_saved_pref_src ;
  }
  else {
    string_lst = ToPrefixesList ;
    nb = &ToPrefixesNumber ;
    list = BottomPrefList ;
    trigger = BottomPrefTrigger ;
    unit_name = LstGetSelectionText (BottomList,
				     LstGetSelection (BottomList)) ;
    last_used_prefix = last_saved_pref_dest ;
  }

  /* Libere la memoire utilisee par l'ancienne liste */
  if (string_lst) MemPtrFree (string_lst) ;

  /* Fabrique la nouvelle liste si il y a une unite selectionnee */
  /* dans le liste des unites. */
  if (unit_name)
    string_lst = PrefixListFromUnitName (unit_name, CurrentCategory, nb) ;
  else {
    *nb = 0 ;
    string_lst = NULL ;
  }

  /* Connecte l'objet liste a la nouvelle liste de chaines */
  LstSetListChoices (list, string_lst, *nb) ;

  /* Positionne la selection sur le premier element ou (si c'est */
  /* la premiere init de l'exec du prog) sur le prefixe present  */
  /* au moment du changement d'application */
  /* On eviter de mettre une selection sur un prefixe inexistant */
  if (last_used_prefix >= *nb) last_used_prefix = 0 ;
  LstSetSelection (list, last_used_prefix) ;

  /* Positionne le texte du trigger avec le nom de l'unite actuelle */
  /* si elle existe */
  CtlSetLabel (trigger, LstGetSelectionText (list, last_used_prefix)) ;
}




short PrefixCoefFromName (char *prefix_name)
{
  switch (PREFIX_CHARS_TO_ID (prefix_name[0], prefix_name[1]))
    {
    case PREFIX_ID_ATTO:
      return (PREFIX_COEF_ATTO) ;
      break ;

    case PREFIX_ID_FEMTO:
      return (PREFIX_COEF_FEMTO) ;
      break ;

    case PREFIX_ID_PICO:
      return (PREFIX_COEF_PICO) ;
      break ;

    case PREFIX_ID_NANO:
      return (PREFIX_COEF_NANO) ;
      break ;

    case PREFIX_ID_MICRO:
      return (PREFIX_COEF_MICRO) ;
      break ;

    case PREFIX_ID_MILLI:
      return (PREFIX_COEF_MILLI) ;
      break ;

    case PREFIX_ID_CENTI:
      return (PREFIX_COEF_CENTI) ;
      break ;

    case PREFIX_ID_DECI:
      return (PREFIX_COEF_DECI) ;
      break ;

    case PREFIX_ID_DECA:
      return (PREFIX_COEF_DECA) ;
      break ;

    case PREFIX_ID_HECTO:
      return (PREFIX_COEF_HECTO) ;
      break ;

    case PREFIX_ID_KILO:
      return (PREFIX_COEF_KILO) ;
      break ;

    case PREFIX_ID_MEGA:
      return (PREFIX_COEF_MEGA) ;
      break ;

    case PREFIX_ID_GIGA:
      return (PREFIX_COEF_GIGA) ;
      break ;

    case PREFIX_ID_TERA:
      return (PREFIX_COEF_TERA) ;
      break ;

    case PREFIX_ID_PETA:
      return (PREFIX_COEF_PETA) ;
      break ;

    case PREFIX_ID_EXA:
      return (PREFIX_COEF_EXA) ;
      break ;

    default:
      /* case PREFIX_ID_NONE: */
      return (PREFIX_COEF_NONE) ;
      break ;
    }
}
