/*
Units V1.1 - A general units converter
Copyright (C) 1998 Franois Pessaux (francois.pessaux@inria.fr)


This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>
#include "protos.h"
#include "messages.h"

#include "pilrc.h"
#include "units.h"
#include "bool.h"


extern Word CurrentCategory ;
extern char CurrentCategoryName[dmCategoryLength] ;
extern DmOpenRef unit_db ;
extern char **UnitsLstItems ;
extern Word UnitsNbEntries ;
extern unsigned char selection_continuation ;


UInt record_to_modify_index ;


Boolean SelectUnitEventHandler (EventPtr event)
{
  FormPtr form ;
  ListPtr list ;

  switch (event->eType) 
    {
    case frmOpenEvent:
      form = FrmGetActiveForm () ;
      CategoryGetName (unit_db, CurrentCategory, CurrentCategoryName) ;

      /* Positionne le trigger de categories a la categorie courante */
      CategorySetTriggerLabel (
	  FrmGetObjectPtr (form,
                           FrmGetObjectIndex (form, unitCategoryTrigger2)),
	  CurrentCategoryName) ;

      /* Met a jour la liste d'unites disponibles sans la recalculer */
      list = FrmGetObjectPtr (form,
			      FrmGetObjectIndex (form, unitsAvailable)) ;
      LstSetListChoices (list, UnitsLstItems, UnitsNbEntries) ;
      LstSetSelection (list, 0) ;
      FrmDrawForm (form) ;
      return (TRUE) ;
      break ;

    case ctlSelectEvent:
      form = FrmGetActiveForm () ;
      list = FrmGetObjectPtr (form, FrmGetObjectIndex (form, unitsAvailable)) ;
      switch (event->data.ctlEnter.controlID)
	{
	case unitCategoryTrigger2:
	  /* Si a la place de TRUE on met FALSE pour le flag disant si */
	  /* on est dans la barre de titre, alors la rubrique "Toutes" */
	  /* ne sera plus accessible par le trigger. */
	  CategorySelectV10 (unit_db, form, unitCategoryTrigger2,
			     unitCategoryList2, TRUE, &CurrentCategory,
			     CurrentCategoryName) ;

	  /* On met a jour la liste d'unites */
	  ReleaseUnitList (UnitsLstItems) ;
	  UnitsLstItems = BuildUnitLists (CurrentCategory, &UnitsNbEntries) ;
	  LstSetListChoices (list, UnitsLstItems, UnitsNbEntries) ;
	  LstSetSelection (list, 0) ;

	  /* Force le redessin de la form */
	  FrmDrawForm (form) ;
	  return (TRUE) ;
	  break ;

	case cancelSelectButton:
	  FrmReturnToForm (0) ;
	  GlobalUpdateMainFormUI () ;
	  return (TRUE) ;
	  break ;

	case okSelectButton:
	  /* Verifie l'existence du record et recupere par effet de bord */
	  /* son index */
	  if (! FindUnitRecordIndexByName (
		     LstGetSelectionText (list, LstGetSelection (list)),
		     &record_to_modify_index, CurrentCategory)) {
	    FrmCustomAlert (standardErrorAlert, MSG_UNOTFOUND, NULL, NULL) ;
	    /* On ne fait rien */
	    return (TRUE) ;
	  }
	  FrmReturnToForm (0) ;

	  /* On continue en fonction de ce pour quoi le */
	  /* mecanisme de selection a ete appele. */
	  switch (selection_continuation)
	    {
	    case CONTINUATION_REMOVE:
	      if (FrmCustomAlert (standardYesNoAlert,
				  MSG_DELETE, NULL, NULL) == 0) {
		/* Delete plutot que Remove, pour la HotSync */
		DmDeleteRecord (unit_db, record_to_modify_index) ;
	      }
	      /* Faut reconstruire la liste des unites */
	      ReleaseUnitList (UnitsLstItems) ;
	      UnitsLstItems = BuildUnitLists(
		                CurrentCategory, &UnitsNbEntries) ;
	      GlobalUpdateMainFormUI () ;
	      break ;

	    case CONTINUATION_MODIFY:
	      FrmPopupForm (definitionUnitForm) ;
	      break ;
	    }
	  return (TRUE) ;
	  break ;
	}
      break ;
    }

return (FALSE) ;
}
