/*
Units V1.1 - A general units converter
Copyright (C) 1998 Franois Pessaux (francois.pessaux@inria.fr)


This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>
#include "protos.h"
#include "gccfloats.h"


#include "pilrc.h"
#include "units.h"
#include "bool.h"
#include "messages.h"


extern DmOpenRef unit_db ;



/* Recree les listes d'unites, met a jour les pointeurs   */
/* de liste des selecteurs, efface la selection de la liste */
/* et efface le contenu des popups. */
char** BuildUnitLists (Word category, Word *nbentries)
{
  UInt index ;
  UInt i ;
  VoidHand handle ;
  VoidPtr record ;
  Word found_entries ;
  char **string_array ;


  found_entries = DmNumRecordsInCategory (unit_db, category) ;
  if (found_entries != 0) {
    /* On alloue le tableau directement locke */
    string_array = (char**) MemPtrNew (found_entries*sizeof(char*)) ;
    if (!string_array) {
      FrmCustomAlert (standardErrorAlert, MSG_NO_MEM, NULL, NULL) ;
      return (NULL) ;
    }

    /* Init des pointeurs du tableau */
    for (i=0, index=0; i<found_entries; i++, index++) {
      handle = DmQueryNextInCategory (unit_db, &index, category) ;
      if (!handle) {
	FrmCustomAlert (standardErrorAlert, MSG_CANT_GET_RECORD, NULL, NULL) ;
	return (NULL) ;
      }

      record = MemHandleLock (handle) ;
      if (!record) {
	FrmCustomAlert (standardErrorAlert, MSG_CANT_GET_RECORD, NULL, NULL) ;
	return (NULL) ;
      }

      string_array[i] =
	          MemPtrNew(((struct UnitRecordHeader*)record)->namesize) ;
      StrCopy (string_array[i], &((char*)record)[nameDataOffset]) ;
      MemHandleUnlock (handle) ;
      DmReleaseRecord (unit_db, index, FALSE) ;
    }
  }
  else {
    /* If no entries don't do anything */
    string_array = NULL ;
  }

  if (nbentries) *nbentries = found_entries ;
  return (string_array) ;
}



void ReleaseUnitList (char **string_array)
{
  UInt found_entries ;
  UInt i ;

  /* Taille du tableau = taille totale / taille d'une case */
  if (string_array) {
    found_entries = (UInt) ((MemPtrSize (string_array)) / sizeof(char*)) ;
    if (string_array) {
      for (i=0;i<found_entries; i++) MemPtrFree (string_array[i]) ;
      MemPtrFree (string_array) ;
      string_array = NULL ;
    }
  }
}

