/*
Units V1.1 - A general units converter
Copyright (C) 1998 Franois Pessaux (francois.pessaux@inria.fr)


This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include <Common.h>
#include <System/SysAll.h>
#define _DONT_USE_FP_TRAPS_ 1
#include <System/NewFloatMgr.h>
#include <UI/UIAll.h>
#include "protos.h"
#include "gccfloats.h"

#include "pilrc.h"
#include "units.h"
#include "bool.h"


extern DmOpenRef unit_db ;
extern Word CurrentCategory ;
extern char CurrentCategoryName[dmCategoryLength] ;

extern FormPtr MainForm ;
extern ListPtr TopList, BottomList, TopPrefList, BottomPrefList ;
extern ControlPtr CategoryTrigger, TopTrigger, BottomTrigger,
                  TopPrefTrigger, BottomPrefTrigger ;
extern char **FromPrefixesList ;
extern Word UnitsNbEntries ;
extern char **UnitsLstItems ;
extern unsigned char selection_continuation ;
extern FieldPtr TopField ;
extern Word last_saved_src, last_saved_dest ;
extern Boolean pretty_floats ;
extern ControlPtr ScientNotChkbox ;
extern Word last_saved_pref_src, last_saved_pref_dest ;



void GlobalUpdateMainFormUI (void)
{
  /* Refresh des UI-Objs listes d'unites */
  LstSetListChoices (TopList, UnitsLstItems, UnitsNbEntries) ;
  LstSetSelection (TopList, last_saved_src) ;
  LstSetListChoices (BottomList, UnitsLstItems, UnitsNbEntries) ;
  LstSetSelection (BottomList, last_saved_dest) ;
  /* Triggers des unites disponibles */
  CtlSetLabel (TopTrigger,
	       LstGetSelectionText (TopList, last_saved_src)) ;
  CtlSetLabel (BottomTrigger,
	       LstGetSelectionText (BottomList, last_saved_dest)) ;
  /* Liste des prefixes disponibles */
  RefreshPrefixList (unitFromTrigger) ;
  RefreshPrefixList (unitToTrigger) ;
  /* Popup trigger avec la categorie courante */
  CategoryGetName (unit_db, CurrentCategory, CurrentCategoryName) ;
  CategorySetTriggerLabel (CategoryTrigger, CurrentCategoryName) ;
}



Boolean MainEventHandler (EventPtr event)
{

  switch (event->eType) 
    {
    case frmOpenEvent:
      /* On doit "initialiser" le popup trigger avec la categorie courante */
      CategoryGetName (unit_db, CurrentCategory, CurrentCategoryName) ;
      CategorySetTriggerLabel (CategoryTrigger, CurrentCategoryName) ;
      /* Init des listes d'unites */
      UnitsLstItems = BuildUnitLists (CurrentCategory, &UnitsNbEntries) ;
      GlobalUpdateMainFormUI () ;
      /* Only serve the first time we open the form */
      last_saved_src = 0 ;
      last_saved_dest = 0 ;
      last_saved_pref_src = 0 ;
      last_saved_pref_dest = 0 ;
      /* Set pretty printing checkbox */
      CtlSetValue (ScientNotChkbox, !pretty_floats) ;
      /* On rafraichit la fenetre */
      FrmDrawForm (MainForm) ;
      /* On met le focus dans le field */
      FrmSetFocus (MainForm, FrmGetObjectIndex (MainForm, iHaveField)) ;
      return (TRUE) ;
      break ;


    case ctlSelectEvent:
      switch (event->data.ctlEnter.controlID)
	{
	case unitCategoryTrigger:
	  CategorySelectV10 (unit_db, MainForm, unitCategoryTrigger,
			     unitCategoryList, TRUE, &CurrentCategory,
			     CurrentCategoryName) ;
	  /* On met a jour les listes d'unites */
	  ReleaseUnitList (UnitsLstItems) ;
	  UnitsLstItems = BuildUnitLists (CurrentCategory, &UnitsNbEntries) ;
	  GlobalUpdateMainFormUI () ;
     	  return (TRUE) ;
	  break ;

	case scientNotationChk:
	  pretty_floats = (Boolean) !CtlGetValue (ScientNotChkbox) ;
     	  return (TRUE) ;
	  break ;

	case unitConvertButton:
	  Convert () ;
	  return (TRUE) ;
	  break ;
	}
      break ;

    case popSelectEvent:
      /* event->data.popSelect.selection : Entree selectionnee */ 
      /* event->data.popSelect.controlID : id du bouton selectionne */
      switch (event->data.popSelect.controlID)
	{
	  /* Si c'est un trigger d'unite, il faut mettre */
	  /* a jour la liste de prefixes autorises */
	case unitFromTrigger:
	case unitToTrigger:
	  RefreshPrefixList (event->data.popSelect.controlID) ;
	  break ;
      }
      /* Laisse le systeme finir le mise a jour du trigger */
      return (FALSE) ;
      break ;


    case menuEvent:
      switch (event->data.menu.itemID)
	{
	/* Cut et Paste ne marchent que sur le field editable */
	case menuitemCut:
	case menuitemPaste:
	  if (FrmGetFocus (MainForm) ==
	      FrmGetObjectIndex (MainForm, iHaveField)) {
	    switch (event->data.menu.itemID) {
	    case menuitemCut: FldCut (TopField) ; break ;
	    case menuitemPaste: FldPaste (TopField) ; break ; }
	  }
	  return (TRUE) ;
	  break ;

	/* Copy marche aussi avec le field non editable */
	case menuitemCopy:
	  { FieldPtr tmp_fld ;
	    Word tmp_index ;
	  
	    tmp_index = FrmGetFocus (MainForm) ;
	    if (tmp_index != -1) {
	      tmp_fld = FrmGetObjectPtr (MainForm, tmp_index) ;
	      FldCopy (tmp_fld) ;
	    }
	  }
	  return (TRUE) ;
	  break ;

	case menuitemNewUnit:
	  selection_continuation = CONTINUATION_NONE ;
	  FrmPopupForm (definitionUnitForm) ;
	  return (TRUE) ;
	  break ;

	case menuitemModUnit:
	  selection_continuation = CONTINUATION_MODIFY ;
	  FrmPopupForm (selectUnitForm) ;
	  return (TRUE) ;
	  break ;

	case menuitemRemUnit:
	  selection_continuation = CONTINUATION_REMOVE ;
	  FrmPopupForm (selectUnitForm) ;
	  return (TRUE) ;

	case menuitemApropos:
	  FrmCustomAlert (standardInfoAlert,
			  "Units V1.11 1998",
			  "\251 Francois PESSAUX",
			  "Freeware of course !") ;
	  return (TRUE) ;
	  break ;
	}
    }
  return (FALSE) ;
}

