/*
Units V1.1 - A general units converter
Copyright (C) 1998 Franois Pessaux (francois.pessaux@inria.fr)


This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#define CAT_APP_ID 'UniT'
#define CAT_DB_TYPE 'Data'
#define DB_NAME "UnitsCategory"

#define factAOffset 0
#define factBOffset (sizeof(double))
#define factMOffset  (factBOffset+(sizeof(double)))
#define prefixesOffset (factMOffset+(sizeof(double)))
#define nameSizeOffset (prefixesOffset+(sizeof(unsigned short)))
#define nameDataOffset (nameSizeOffset+(sizeof(unsigned char)))

struct UnitRecordHeader {
  double afactor ;
  double bfactor ;
  double mfactor ;
  unsigned short prefixes ;
  unsigned char namesize ;
};

/* Utile pour expanser les constantes dans le switch de conversion */
/* Si on le fait a partie des chaines, Gcc ne se rend pas compte que */
/* ca se reduit sur une constante ! */
#define PREFIX_CHARS_TO_ID(c1, c2) \
     ((c1 << 8) | c2)

#define PREFIX_COEF_ATTO (-18)
#define PREFIX_COEF_FEMTO (-15)
#define PREFIX_COEF_PICO (-12)
#define PREFIX_COEF_NANO (-9)
#define PREFIX_COEF_MICRO (-6)
#define PREFIX_COEF_MILLI (-3)
#define PREFIX_COEF_CENTI (-2)
#define PREFIX_COEF_DECI (-1)
#define PREFIX_COEF_DECA (1)
#define PREFIX_COEF_HECTO (2)
#define PREFIX_COEF_KILO (3)
#define PREFIX_COEF_MEGA (6)
#define PREFIX_COEF_GIGA (9)
#define PREFIX_COEF_TERA (12)
#define PREFIX_COEF_PETA (15)
#define PREFIX_COEF_EXA (18)
#define PREFIX_COEF_NONE (0)

#define PREFIX_ID_ATTO PREFIX_CHARS_TO_ID('a', ' ')
#define PREFIX_ID_FEMTO PREFIX_CHARS_TO_ID('f', ' ')
#define PREFIX_ID_PICO PREFIX_CHARS_TO_ID('p', ' ')
#define PREFIX_ID_NANO PREFIX_CHARS_TO_ID('n', ' ')
/* \265 = char code 181 = mu */
#define PREFIX_ID_MICRO PREFIX_CHARS_TO_ID('\265', ' ')
#define PREFIX_ID_MILLI PREFIX_CHARS_TO_ID('m', ' ')
#define PREFIX_ID_CENTI PREFIX_CHARS_TO_ID('c', ' ')
#define PREFIX_ID_DECI PREFIX_CHARS_TO_ID('d', ' ')
#define PREFIX_ID_DECA PREFIX_CHARS_TO_ID('d', 'a')
#define PREFIX_ID_HECTO PREFIX_CHARS_TO_ID('h', ' ')
#define PREFIX_ID_KILO PREFIX_CHARS_TO_ID('k', ' ')
#define PREFIX_ID_MEGA PREFIX_CHARS_TO_ID('M', ' ')
#define PREFIX_ID_GIGA PREFIX_CHARS_TO_ID('G', ' ')
#define PREFIX_ID_TERA PREFIX_CHARS_TO_ID('T', ' ')
#define PREFIX_ID_PETA PREFIX_CHARS_TO_ID('P', ' ')
#define PREFIX_ID_EXA PREFIX_CHARS_TO_ID('E', ' ')
#define PREFIX_ID_NONE PREFIX_CHARS_TO_ID(' ', ' ')

#define CONTINUATION_NONE 0
#define CONTINUATION_REMOVE 1
#define CONTINUATION_MODIFY 2


struct UnitPreferences {
  /* Current catgory ID */
  Word current_category ;
  /* Flag to indicate if src_unique_id and dest_unique_id are significant */
  /* They are not in case where to units were selected while switching    */
  /* application. This occurs if the database or the current category is  */
  /* empty. */
  Boolean ids_are_valid ;
  /* Unique ID for record containing source unit */
  ULong src_unique_id ;
  /* Unique ID for record containing destionation unit */
  ULong dest_unique_id ;
  Word src_prefix ;
  Word dest_prefix ;
  /* Tag telling if me must attempt pretty printing for floats */
  Boolean pretty_floats ;
};
